package org.robokind.headless.animation;

import java.util.logging.Logger;
import javax.jms.Connection;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;
import org.robokind.api.animation.library.AnimationLibrary;
import org.robokind.api.animation.library.DefaultAnimationLibrary;
import org.robokind.api.animation.lifecycle.AnimationPlayerHostLifecycle;
import org.robokind.api.animation.protocol.AnimationEvent;
import org.robokind.avrogen.animation.AnimationEventRecord;
import org.robokind.api.common.osgi.OSGiUtils;
import org.robokind.api.common.osgi.lifecycle.OSGiComponent;
import org.robokind.impl.animation.messaging.PortableAnimationEvent;
import org.robokind.impl.messaging.lifecycle.JMSAvroAsyncReceiverLifecycle;
import org.robokind.impl.messaging.utils.ConnectionManager;
import org.robokind.impl.messaging.utils.ConnectionUtils;

public class Activator implements BundleActivator {
    private final static Logger theLogger = 
            Logger.getLogger(Activator.class.getName());
    public final static String PLAYER_ID = "myRobot";
    public final static String ANIM_RECEIVER_ID = "animationReceiver";
    public final static String CONNECTION_ID = "animationConnection";
    public final static String ANIM_DEST_ID = "animationRequest";
    public final static String LIBRARY_ID = "myRobot";
    
    
    @Override
    public void start(BundleContext context) throws Exception {
        connectAnimation(context);
        launchRemotePlayer(context, PLAYER_ID, ANIM_RECEIVER_ID,
                CONNECTION_ID, ANIM_DEST_ID);
        new OSGiComponent(context, new AnimationPlayerHostLifecycle(
                "Avatar_ZenoR50", ANIM_RECEIVER_ID)
        ).start();
        launchAnimationLibrary(context, LIBRARY_ID);
    }
    
    private void connectAnimation(BundleContext context) throws Exception {
        Connection con = ConnectionManager.createConnection(
                "admin", "admin", "client1", "test", 
                "tcp://127.0.0.1:5672");
        con.start();
        theLogger.info("Registering Animation Connection and Destinations");
        ConnectionUtils.ensureSession(context, 
                "animationConnection", con, null);
        ConnectionUtils.ensureDestinations(context, 
                "animationRequest", "animationRequest", ConnectionUtils.QUEUE, null);
        theLogger.info("Animation Connection and Destinations Registered");
    }
    
    private void launchRemotePlayer(BundleContext context, 
            String playerId, String receiverId, String conId, String destId){
        theLogger.info("Launching Dynamic RemoteAnimationPlayerHost Service.");
        new OSGiComponent(context, 
                new AnimationPlayerHostLifecycle(playerId, receiverId)
        ).start();
        
        JMSAvroAsyncReceiverLifecycle reqRecLifecycle = 
                new JMSAvroAsyncReceiverLifecycle(
                        new PortableAnimationEvent.RecordMessageAdapter(), 
                        AnimationEvent.class, AnimationEventRecord.class, 
                        AnimationEventRecord.SCHEMA$, ANIM_RECEIVER_ID, 
                        conId, destId);
        OSGiComponent reqRec = new OSGiComponent(context, reqRecLifecycle);
        reqRec.start();
        theLogger.info("Dynamic RemoteAnimationPlayerHost Service Launched.");
    }
    
    private void launchAnimationLibrary(BundleContext context, String libraryId){
        theLogger.info("Launching AnimationLibrary Service.");
        AnimationLibrary library = new DefaultAnimationLibrary(libraryId);
        ServiceRegistration reg = OSGiUtils.registerService(context, 
                AnimationLibrary.class.getName(), 
                AnimationLibrary.PROP_ANIM_PLAYER_ID, 
                libraryId, library, null);
        theLogger.info("AnimationLibrary Service Launched.");
    }

    @Override
    public void stop(BundleContext context) throws Exception {
        // TODO add deactivation code here
    }

}
